
pragma solidity *;

pragma experimental ABIEncoderV2;

import "@openzeppelin/contracts/token/ERC20/ERC20.sol" as ERC20;
import "@openzeppelin/contracts/math/SafeMath.sol";
import "@openzeppelin/contracts/math/Math.sol";
import "@openzeppelin/contracts/access/Ownable.sol";

import { ReentrancyGuard } from "@openzeppelin/contracts/utils/ReentrancyGuard.sol";
import { IERC20 } from "@openzeppelin/contracts/token/ERC20/IERC20.sol";
import { SafeERC20 } from "@openzeppelin/contracts/token/ERC20/SafeERC20.sol";

import { ITier } from "../tier/ITier.sol";

enum Phase {
    ZERO,
    ONE,
    TWO,
    THREE,
    FOUR,
    FIVE,
    SIX,
    SEVEN,
    EIGHT
}

import { RedeemableERC20, RedeemableERC20Config } from "../redeemableERC20/RedeemableERC20.sol";

import { RedeemableERC20Pool, RedeemableERC20PoolConfig } from "../pool/RedeemableERC20Pool.sol";
import { SeedERC20, SeedERC20Config } from "../seed/SeedERC20.sol";

import { RedeemableERC20Factory } from "../redeemableERC20/RedeemableERC20Factory.sol";


struct TrustContracts {
    address reserveERC20;
    address redeemableERC20;
    address redeemableERC20Pool;
    address seeder;
    address tier;
    address crp;
    address pool;
}

enum DistributionStatus {
    Pending,
    Seeded,
    Trading,
    TradingCanEnd,
    Success,
    Fail
}

struct DistributionProgress {
    DistributionStatus distributionStatus;
    uint32 distributionStartBlock;
    uint32 distributionEndBlock;
    uint256 poolReserveBalance;
    uint256 poolTokenBalance;
    uint256 reserveInit;
    uint256 minimumCreatorRaise;
    uint256 seederFee;
    uint256 redeemInit;
}

struct TrustConfig {
    address creator;
    uint256 minimumCreatorRaise;
    address seeder;
    uint256 seederFee;
    uint16 seederUnits;
    uint16 seederCooldownDuration;
    uint256 redeemInit;
}

struct TrustRedeemableERC20Config {
    RedeemableERC20Factory redeemableERC20Factory;
    string name;
    string symbol;
    uint256 totalSupply;
}

struct TrustRedeemableERC20PoolConfig {
    IERC20 reserve;
    uint256 reserveInit;
    uint256 initialValuation;
    uint256 finalValuation;
    uint256 minimumTradingDuration;
}

contract Trust is ReentrancyGuard {

    using SafeMath for uint256;
    using Math for uint256;

    using SafeERC20 for IERC20;
    using SafeERC20 for RedeemableERC20;

    address public creator;
    uint256 public minimumCreatorRaise;
    address public seeder;
    uint256 public seederFee;
    uint16 public seederUnits;
    uint16 public seederCooldownDuration;
    uint256 public redeemInit;
    uint256 public finalBalance;
    uint256 public successBalance;
    RedeemableERC20 public token;
    RedeemableERC20Pool public pool;

    constructor (
        TrustConfig memory config_,
        TrustRedeemableERC20Config memory trustRedeemableERC20Config_,
        TrustRedeemableERC20PoolConfig memory trustRedeemableERC20PoolConfig_
    ) public {
    }

    function getContracts() external view returns(TrustContracts memory) {
        return TrustContracts(
            address(pool.reserve()),
            address(token),
            address(pool),
            address(seeder),
            address(token.tierContract()),
            address(pool.crp()),
            address(pool.crp().bPool())
        );
    }

    
    function getDistributionProgress()
        external
        view
        returns(DistributionProgress memory)
    {
        address balancerPool_ = address(pool.crp().bPool());
        uint256 poolReserveBalance_;
        uint256 poolTokenBalance_;
        if (balancerPool_ != address(0)) {
            poolReserveBalance_ = pool.reserve().balanceOf(balancerPool_);
            poolTokenBalance_ = token.balanceOf(balancerPool_);
        }
        else {
            poolReserveBalance_ = 0;
            poolTokenBalance_ = 0;
        }

        return DistributionProgress(
            getDistributionStatus(),
            pool.phaseBlocks(0),
            pool.phaseBlocks(1),
            poolReserveBalance_,
            poolTokenBalance_,
            pool.reserveInit(),
            minimumCreatorRaise,
            seederFee,
            redeemInit
        );
    }

    
    function getDistributionStatus() public view returns (DistributionStatus) {
        Phase poolPhase_;
        if (poolPhase_ == Phase.ZERO) {
            if ( true) {
                return DistributionStatus.Seeded;
            } else {
                return DistributionStatus.Pending;
            }
        }
    }
}
